use sqlx::{Pool, Postgres};

use crate::AppError;

pub struct Account {
    pub account_id: String,
}

pub struct CaughtFish {
    pub id: Option<i32>,
    pub account_id: String,
    pub fish_id: i32,
}

pub async fn insert_account(id: &str, pool: &Pool<Postgres>) -> Result<(), AppError> {
    sqlx::query!("INSERT INTO accounts (account_id) VALUES ($1)", id)
        .execute(pool)
        .await?;
    Ok(())
}

pub async fn get_account(id: &str, pool: &Pool<Postgres>) -> Result<bool, AppError> {
    let results = sqlx::query_as!(Account, "SELECT * FROM accounts WHERE account_id = $1", id)
        .fetch_all(pool)
        .await?;

    if results.len() == 1 {
        Ok(true)
    } else {
        Ok(false)
    }
}

pub async fn insert_caught_fish(
    acc_id: &str,
    fish_id: &i32,
    pool: &Pool<Postgres>,
) -> Result<(), AppError> {
    sqlx::query!(
        "
        INSERT INTO caught_fish (account_id, fish_id) VALUES ($1, $2)
        ",
        acc_id,
        fish_id
    )
    .execute(pool)
    .await?;

    Ok(())
}

pub async fn get_caught_fish(acc_id: &str, pool: &Pool<Postgres>) -> Result<Vec<i32>, AppError> {
    let results = sqlx::query_as!(
        CaughtFish,
        "SELECT * FROM caught_fish WHERE account_id = $1",
        acc_id
    )
    .fetch_all(pool)
    .await?;

    Ok(results.iter().map(|cf| cf.fish_id).collect())
}
